@@ -7,8 +7,12 @@
 #ifndef LVM_ACTIVATE_H
 #define LVM_ACTIVATE_H
 
+#include "metadata.h"
 #include <libdevmapper.h>
 
+void set_activation(int activation);
+int activation();
+
 int driver_version(char *version, size_t size);
 int library_version(char *version, size_t size);
 
@@ -16,6 +20,10 @@ int library_version(char *version, size_t size);
  * Returns 1 if info structure has been populated, else 0.
  */
 int lv_info(struct logical_volume *lv, struct dm_info *info);
+/*
+ * Returns 1 if percent has been set, else 0.
+ */
+int lv_snapshot_percent(struct logical_volume *lv, float *percent);
 
 /*
  * These should eventually use config file
