@@ -77,6 +77,12 @@ static int _read_name_params(struct lvcreate_params *lp,
 			}
 
 		} else {
+			if (strrchr(argv[0], '/')) {
+				log_error("Volume group name expected "
+					  "(no slash)");
+				return 0;
+			}
+
 			/*
 			 * Ensure lv_name doesn't contain a
 			 * different VG.
@@ -84,7 +90,7 @@ static int _read_name_params(struct lvcreate_params *lp,
 			if (lp->lv_name && strchr(lp->lv_name, '/')) {
 				if (!(lp->vg_name =
 				      extract_vgname(cmd, lp->lv_name)))
-					    return 0;
+					return 0;
 
 				if (strcmp(lp->vg_name, argv[0])) {
 					log_error("Inconsistent volume group "
@@ -103,6 +109,13 @@ static int _read_name_params(struct lvcreate_params *lp,
 	if (lp->lv_name && (ptr = strrchr(lp->lv_name, '/')))
 		lp->lv_name = ptr + 1;
 
+	/* FIXME Remove this restriction eventually */
+	if (lp->lv_name && !strncmp(lp->lv_name, "snapshot", 8)) {
+		log_error("Names starting \"snapshot\" are reserved. "
+			  "Please choose a different LV name.");
+		return 0;
+	}
+
 	return 1;
 }
 
@@ -188,12 +201,24 @@ static int _read_stripe_params(struct lvcreate_params *lp,
 static int _read_params(struct lvcreate_params *lp, struct cmd_context *cmd,
 			int argc, char **argv)
 {
-	/*
-	 * Set the defaults.
-	 */
 	memset(lp, 0, sizeof(*lp));
-	lp->chunk_size = 2 * arg_int_value(cmd, chunksize_ARG, 8);
-	log_verbose("setting chunksize to %d sectors.", lp->chunk_size);
+
+	/*
+	 * Check selected options are compatible and set defaults
+	 */
+	if (arg_count(cmd, snapshot_ARG)) {
+		if (arg_count(cmd, zero_ARG)) {
+			log_error("-s and -Z are incompatible");
+			return 0;
+		}
+		lp->chunk_size = 2 * arg_int_value(cmd, chunksize_ARG, 8);
+		log_verbose("Setting chunksize to %d sectors.", lp->chunk_size);
+	} else {
+		if (arg_count(cmd, chunksize_ARG)) {
+			log_error("-c is only available with -s");
+			return 0;
+		}
+	}
 
 	if (!_read_name_params(lp, cmd, &argc, &argv) ||
 	    !_read_size_params(lp, cmd, &argc, &argv) ||
@@ -294,21 +319,21 @@ static int _lvcreate(struct cmd_context *cmd, struct lvcreate_params *lp)
 {
 	uint32_t size_rest;
 	uint32_t status = 0;
+	alloc_policy_t alloc = ALLOC_DEFAULT;
 	struct volume_group *vg;
 	struct logical_volume *lv, *org;
 	struct list *pvh;
+	int consistent = 1;
 
 	if (lp->contiguous)
-		status |= ALLOC_CONTIGUOUS;
-	else
-		status |= ALLOC_SIMPLE;
+		alloc = ALLOC_CONTIGUOUS;
 
-	status |= lp->permission;
+	status |= lp->permission | VISIBLE_LV;
 
 	/* does VG exist? */
 	log_verbose("Finding volume group \"%s\"", lp->vg_name);
 
-	if (!(vg = vg_read(cmd, lp->vg_name))) {
+	if (!(vg = vg_read(cmd, lp->vg_name, &consistent))) {
 		log_error("Volume group \"%s\" doesn't exist", lp->vg_name);
 		return 0;
 	}
@@ -374,14 +399,30 @@ static int _lvcreate(struct cmd_context *cmd, struct lvcreate_params *lp)
 		lp->extents = lp->extents - size_rest + lp->stripes;
 	}
 
-	if (lp->snapshot && !(org = find_lv(vg, lp->origin))) {
-		log_err("Couldn't find origin volume '%s'.", lp->origin);
+	if (!activation()) {
+		if (lp->snapshot)
+			log_error("Can't create snapshot without using "
+				  "device-mapper kernel driver");
 		return 0;
 	}
 
-	if (!(lv = lv_create(vg->fid, lp->lv_name, status,
+	if (lp->snapshot) {
+		if (!(org = find_lv(vg, lp->origin))) {
+			log_err("Couldn't find origin volume '%s'.",
+				lp->origin);
+			return 0;
+		}
+		if (lv_is_cow(org)) {
+			log_error("Snapshots of snapshots are not supported "
+				  "yet.");
+			return 0;
+		}
+	}
+
+	if (!(lv = lv_create(vg->fid, lp->lv_name, status, alloc,
 			     lp->stripes, lp->stripe_size, lp->extents,
-			     vg, pvh))) return 0;
+			     vg, pvh)))
+		return 0;
 
 	if (lp->read_ahead) {
 		log_verbose("Setting read ahead sectors");
@@ -398,16 +439,28 @@ static int _lvcreate(struct cmd_context *cmd, struct lvcreate_params *lp)
 		return 0;
 
 	/* store vg on disk(s) */
-	if (!vg_write(vg))
+	if (!vg_write(vg)) {
 		return 0;
+	}
 
-	if (!lock_vol(cmd, lv->lvid.s, LCK_LV_ACTIVATE))
+	if (!lock_vol(cmd, lv->lvid.s, LCK_LV_ACTIVATE)) {
+		/* FIXME Remove the failed lv we just added */
+		log_error("Aborting. Failed to wipe snapshot "
+			  "exception store. Remove new LV and retry.");
 		return 0;
+	}
 
-	if (lp->zero || lp->snapshot)
-		_zero_lv(cmd, lv);
-	else
-		log_print("WARNING: \"%s\" not zeroed", lv->name);
+	if ((lp->zero || lp->snapshot) && activation()) {
+		if (!_zero_lv(cmd, lv) && lp->snapshot) {
+			/* FIXME Remove the failed lv we just added */
+			log_error("Aborting. Failed to wipe snapshot "
+				  "exception store. Remove new LV and retry.");
+			return 0;
+		}
+	} else {
+		log_error("WARNING: \"%s\" not zeroed", lv->name);
+		/* FIXME Remove the failed lv we just added */
+	}
 
 	if (lp->snapshot) {
 		if (!lock_vol(cmd, lv->lvid.s, LCK_LV_DEACTIVATE)) {
@@ -420,7 +473,7 @@ static int _lvcreate(struct cmd_context *cmd, struct lvcreate_params *lp)
 			return 0;
 		}
 
-		if (!vg_add_snapshot(org, lv, 1, lp->chunk_size)) {
+		if (!vg_add_snapshot(org, lv, 1, NULL, lp->chunk_size)) {
 			log_err("Couldn't create snapshot.");
 			return 0;
 		}
